#!/bin/bash
#

# buildPqxx.sh <product_dir> <e5|e6> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e5|e6> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    PRODUCTS=${product_dir}:${PRODUCTS}
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version} from ${product_dir}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pqxx
origpkgver=v4_0_1
pkgver=${origpkgver}e
ssibuildshims_version=v0_08_04
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//' | sed -e 's/a//'`
pkgtarname=libpqxx-${pkgdotver}
pgsqlver=v9_1_15

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == "e[245]" ]]
then
  cxxflg="${cflg} -std=c++11 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e6" ]]
then
  cxxflg="${cflg} -std=c++1y -Wno-deprecated-declarations"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

which gcc

if [ -z ${POSTGRESQL_VERSION} ]
then
   echo "ERROR: failed to setup postgresql"
   exit 1
else
   echo "building with postgresql version: ${POSTGRESQL_VERSION}"
fi

echo "Building ${package} in ${pkgdir}"

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

set -x


cd ${sourcedir}
tar -xf ${tardir}/${pkgtarname}.tar.gz
# apply patch 
patch -b -p0 < ${patchdir}/${pkgtarname}.patch || die "ERROR: failed to apply patch."

cd ${blddir}

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

CXXFLAGS="${cxxflg}" CFLAGS="${cflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} --enable-shared || die "ERROR: configure failed."
make -j ${ncore} || die "ERROR: make failed."
# make check || die "ERROR: make check failed."
make install || die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup ${package} ${pkgver} -q ${fullqual} failed"
echo "${package} is installed at ${PQXX_FQ_DIR}"

# fixing embedded path
mv ${PQXX_FQ_DIR}/bin/pqxx-config ${PQXX_FQ_DIR}/bin/pqxx-config.bak
cat ${PQXX_FQ_DIR}/bin/pqxx-config.bak | sed 's%^prefix=.*$%prefix=${PQXX_FQ_DIR}%' > ${PQXX_FQ_DIR}/bin/pqxx-config
chmod +x  ${PQXX_FQ_DIR}/bin/pqxx-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

exit 0
